import requests
from circuitbreaker import circuit

from zainpay.utils.constants import REQUEST_METHODS


class APIClient:

    base_url: str | None

    def __init__(self, base_url: str | None = None) -> None:
        self.base_url = base_url

    def join_url(self, *urls: str | None) -> str:
        _urls: list[str] = []

        for url in urls:
            if url is not None:
                if url.startswith("/"):
                    url = url[1:]
                if url.endswith("/"):
                    url = url[:-1]
                _urls.append(url)

        return "/".join(_urls) + "/"

    @circuit
    def send_request(
        self,
        method: REQUEST_METHODS,
        url: str,
        headers: dict | None = None,
        data: dict | None = None,
        *args: list | tuple,
        **kwargs: dict,
    ) -> requests.Response:
        try:
            url = self.join_url(self.base_url, url)

            return requests.request(
                method,
                url,
                headers=headers,
                data=data,
                *args,
                **kwargs,
            )
        except Exception as error:
            raise error



import pydantic

from zainpay.utils.api_client.client import APIClient
from zainpay.utils.api_client.factory import APIClientFactory
from zainpay.utils.constants import MODES, REQUEST_METHODS


class ZainBox:

    """
    A zainbox is a virtual bucket that allows a merchant to create unlimited multiple virtual accounts.
    """

    pub_key: str
    api_client: APIClient
    headers: dict

    def __init__(self, pub_key: str, mode: MODES = MODES.DEVELOPMENT) -> None:
        self.pub_key = pub_key
        self.api_client = APIClientFactory(mode=mode)
        self.headers = {
            "Authorization": "Bearer %s" % self.pub_key,
            "Content-Type": "application/json",
        }

    # TODO email param -> use `pydantic.EmailStr` requires `email-validator` package
    @pydantic.validate_arguments()
    def create_zainbox(
        self,
        name: str,
        tags: str,
        codeNamePrefix: str,
        callbackUrl: pydantic.AnyHttpUrl,
        email: str,
    ):
        """
        Create a zainbox.
        """

        _create_zainbox_request = self.api_client.send_request(
            method=REQUEST_METHODS.POST,
            url="/zainbox/create/request",
            headers=self.headers,
            data={
                "name": name,
                "tags": tags,
                "codeNamePrefix": codeNamePrefix,
                "callbackUrl": callbackUrl,
                "email": email,
            },
        )

        return _create_zainbox_request

    def list_zainbox(self):
        """
        Get all your created zainboxes
        """

        _list_zainbox_request = self.api_client.send_request(
            method=REQUEST_METHODS.GET,
            url="/zainbox/list",
            headers=self.headers,
        )

        return _list_zainbox_request



import enum


class REQUEST_METHODS(str, enum.Enum):

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"


class MODES(str, enum.Enum):

    DEVELOPMENT = "DEVELOPMENT"
    PRODUCTION = "PRODUCTION"


DEVELOPMENT_URL: str = "https://sandbox.zainpay.ng/"
PRODUCTION_URL: str = "https://api.zainpay.ng/"



from zainpay.utils.api_client.client import APIClient
from zainpay.utils.constants import DEVELOPMENT_URL, MODES, PRODUCTION_URL


class APIClientFactory(APIClient):
    def __init__(self, mode: MODES = MODES.DEVELOPMENT) -> None:
        base_url: str

        match (mode):
            case MODES.DEVELOPMENT:
                base_url = DEVELOPMENT_URL
            case MODES.PRODUCTION:
                base_url = PRODUCTION_URL
            case _:
                raise NotImplementedError()

        super().__init__(base_url)



from zainpay.utils.constants import MODES
from zainpay.virtual_account.zainbox import ZainBox


class ZainPay:

    zainbox: ZainBox

    def __init__(self, pub_key: str, mode: MODES = MODES.DEVELOPMENT) -> None:
        self.zainbox = ZainBox(pub_key=pub_key, mode=mode)

    def create_zainbox(self, *args, **kwargs):
        return self.zainbox.create_zainbox(*args, **kwargs)

    def list_zainbox(self, *args, **kwargs):
        return self.zainbox.list_zainbox(*args, **kwargs)
